lappend auto_path . 

; Replace dummyPkg with name of package
; Global replace dummyNameSpace with namespace identifier.
; Global replace dummyState with the name of a state array.
; Global Replace dummyWidgetProc with widget procedure name.
; Replace megaDummy with widgetName
; Replace dummy with name of widget.
; Replace scroll list fr with real subwidget names.

package provide dummyPkg 1.0
##############################################################
# proc megaDummy {args}
#    Create a megaDummy megawidget
#    External entry point for creating a megawidget.
# Arguments
#   ?parentFrame?       A frame to use for the parent,
#                       If this is not provided, the mega widget 
#                      is a child of the root frame (".").
# Results
#   Creates a procedure with the megawidget name for processing 
#    widget commands
#   Returns the name of the megawidget

proc megaDummy {args} {
    set newWidget [eval dummyNameSpace::MakemegaDummy $args]
    return $newWidget
}

namespace eval dummyNameSpace {
    # Force the load of the widgetLib package so that the
    #   procedures can be imported.
    
    # Support for Jan Nijtman's wrap

    if {[string match [info command ::wrap::open] ""]} {   
        package require widgetLib
    } else {
        ::wrap::source WidgetLib.tcl
    }

    
    # Two tests, the first catches loading the widgetLib here and now,
    # lsearching the top namespace checks to see if it's already been loaded
    #  by another package, and just needs to be imported to here.

    if {([info exists \
            auto_index(::widgetLib::processOptions)]) ||
        ([lsearch [namespace children ::] ::widgetLib] >= 0) } {
	# Force a load if the package isn't already loaded.
        catch {eval $auto_index(::widgetLib::processOptions)}   
    }

    # Import the procedures to the local namespace.
    # This works with standalone Wish interpreters
    catch {namespace import ::widgetLib::*}

    variable dummyState
    set dummyState(unique) 0
    set dummyState(debug) 0
    
    proc debugPuts {args} {
        variable dummyState
        if {$dummyState(debug)} {
	    eval puts $args
	}
    }
    
    ##############################################################
    # proc MakemegaDummy {args}
    #    Create a megaDummy megawidget
    # Arguments
    #   ?parentFrame?  A frame to use for the parent.
    #                   If this is not provided, the mega widget
    #                  is a child of the root frame (".").
    #   ?-scrollposition left/right?
    #                   The side of the listbox for the scrollbar
    # Results
    #   Returns the name of the parent frame for use as a
    #   megawidget

    proc MakemegaDummy {args} {
        variable dummyState
        #
        # Set some defaults
        #
        set widgetName .scl_$dummyState(unique)
        incr dummyState(unique)

        #
        # If the first argument is a window path, use that as 
        # the base name for this widget.
        #
        if {[string first "." [lindex $args 0]] == 0} {
            set widgetName [lindex $args 0]
            set args [lreplace $args 0 0]
        }
	set f1 $widgetName
	
	#
	# Set Command line option defaults here, 
	#   height, width and local count are freebies

	set dummyState(height) 5;
	set dummyState(width) 20;
	
	set argList {-width width
		-height height
		}
	
        # Scan the args for flags that need special handling
	# height, width  are examples
	#  Remove these entries if they are found.


        foreach {keyName indexName} $argList {
            if {[set pos [lsearch $args $keyName ]] >= 0} {
                set val [expr $pos + 1]
                set dummyState($f1.$indexName) [lindex $args $val]
                set args [lreplace $args $pos $val]
            }
        }
                
        # Create master Frame
        
        set f1 [frame $widgetName -height $dummyState(height) \
	    -width $dummyState(width)]
	
	# Apply invocation options to the master frame, as appropriate

	foreach {opt val} $args {
	   catch {$f1 configure $opt $val}
	}

        ;#v vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv
	; CREATE THE TK WIDGETS HERE, using whatever settings from cmd line

        # EXAMPLE FOR LABEL:
	# label $f1.l -text $dummyState($f1.text)
	# set dummyState(subWidgetName.label) l
	# pack $f1.l -side left
        
	update ; update idle;

        ;#^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^

	debugPuts "PAST WIDGET CREATION"

        # We can't have two widgets with the same name.  
        #  Rename the base frame procedure for this
        #  widget to $widgetName.fr so that we can use
        #  $widgetName as the widget procedure name
        #  for the megawidget.

        uplevel #0 rename $widgetName $widgetName.fr

        proc $widgetName {args} " [namespace code\
          {return \[dummyWidgetProc $widgetName \$args]}] "

        #
        # Set any options that were defined in the command line.
        #
        # Process args as a list to keep from losing the second word of
	#  two word options

        ;#v vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv
	; Any widgets that should accept individual command line options
	;# should be processed in this list
	
	# Example for the previous label

	# set createdWidgetList [list .l]

        processOptions [list $args] $createdWidgetList $widgetName
		
        ;#^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^

	# When this window is destroyed, destroy the associated command.

	bind $widgetName <Destroy> "+[namespace code [list rename $widgetName {}]]"
        return $widgetName
    }
    
    ##########################################################
    #     proc dummyWidgetProc {widgetName args}
    #    The master procedure for handling this megaWidget's
    #    subcommands
    # Arguments
    #   widgetName:          The name of the megaDummy
    #                        widget
    #   args:          The rest of the command line 
    #                               arguments
    #                        The first element must be the
    #                         subcommand name Subsequent list
    #                         elements depend on subcommand
    #     configure - configure entire widget, or certain
    #               subwidgets
    #               (configure subwidgetList -option1 \
    #                     val1 -option2 val2 ...)
    #     subwidget - return the full pathname of a 
    #               requested subwidget
    #     itemconfigure - configure a specific subwidget in this widget.
    #		mega itemconfigure itemID -key value
    #
    # Results
    #   Evaluates a subcommand and returns a result if required.
    #
    proc dummyWidgetProc {widgetName subCommand args} {
        variable dummyState
        set cmdArgs $args
	
	# These are freebies:

	# megawid configure -flag Value ...
	# megawid cget subwidgetName -flag
	# megawid itemconfigure name -flag Value
        # megawid widgetnames << Returns list of names
        # megawid subwidget name << Returns valid name to access this subwidget

        switch -- $subCommand {
        itemconfigure {
                catch {eval $widgetName.$dummyState(subWidgetName.[lindex $cmdArgs 0])  configure [lrange $cmdArgs 1 end]}
		}
        itemcget {
	        if {[llength $cmdArgs != 2]} {
		    error "$widgetName cget subWidgetName -option"
		}
                catch {eval $widgetName.$dummyState(subWidgetName.[lindex $cmdArgs 0])  cget [lindex $cmdArgs 1]}
		}
        configure {
	        foreach n [array names dummyState subWidgetName.*] {
		    lappend names $dummyState($n)
		}
                makeOptionList $cmdArgs $names \
                       subwidgetList options
                set rtn [processOptions $options \
                        $subwidgetList $widgetName]
                return $rtn
            }
        widgetnames {
	        foreach n [array names dummyState subWidgetName.*] {
		    foreach {d name} [split $n .] {}
		    lappend names $name
		}
	        return  $names
             }
        subwidget {
	        set name [lindex $cmdArgs 0]
	        if {[info exists dummyState(subWidgetName.$name)]} {
		    return $widgetName.$dummyState(subWidgetName.$name)
		}
             }
        default {
                error "bad command: $subCommand -\
                    must be configure itemconfigure itemcget widgetnames\
                    or subwidget"
            }
        }
    }
}



if {[info exists argv] &&($argv == "-test")} {

proc testME {} {
    set ht 5 
    set eb [dummy -height $ht -width 35]

    pack $eb
    
}
testME
}

