lappend auto_path . 

package provide sideToggle 1.0
##############################################################
# proc sideToggle {args}
#    Create a sideToggle megawidget
#    External entry point for creating a megawidget.
# Arguments
#   ?parentFrame?       A frame to use for the parent,
#                       If this is not provided, the mega widget 
#                      is a child of the root frame (".").
# Results
#   Creates a procedure with the megawidget name for processing 
#    widget commands
#   Returns the name of the megawidget

proc sideToggle {args} {
    set newWidget [eval sideToggle::MakesideToggle $args]
    return $newWidget
}

namespace eval sideToggle {
    # Force the load of the widgetLib package so that the
    #   procedures can be imported.
    
    # Support for Jan Nijtman's wrap

    if {[string match [info command ::wrap::open] ""]} {   
        package require widgetLib
    } else {
        ::wrap::source WidgetLib.tcl
    }

    
    # Two tests, the first catches loading the widgetLib here and now,
    # lsearching the top namespace checks to see if it's already been loaded
    #  by another package, and just needs to be imported to here.

    if {([info exists \
            auto_index(::widgetLib::processOptions)]) ||
        ([lsearch [namespace children ::] ::widgetLib] >= 0) } {
	# Force a load if the package isn't already loaded.
        catch {eval $auto_index(::widgetLib::processOptions)}   
    }

    # Import the procedures to the local namespace.
    # This works with standalone Wish interpreters
    catch {namespace import ::widgetLib::*}

    variable sideTogState
    set sideTogState(unique) 0
    set sideTogState(debug) 0
    
    proc debugPuts {args} {
        variable sideTogState
        if {$sideTogState(debug)} {
	    eval puts $args
	}
    }
    
    ##############################################################
    # proc MakesideToggle {args}
    #    Create a sideToggle megawidget
    # Arguments
    #   ?parentFrame?  A frame to use for the parent.
    #                   If this is not provided, the mega widget
    #                  is a child of the root frame (".").
    #   ?-scrollposition left/right?
    #                   The side of the listbox for the scrollbar
    # Results
    #   Returns the name of the parent frame for use as a
    #   megawidget

    proc MakesideToggle {args} {
        variable sideTogState
        #
        # Set some defaults
        #
        set widgetName .scl_$sideTogState(unique)
        incr sideTogState(unique)

        #
        # If the first argument is a window path, use that as 
        # the base name for this widget.
        #
        if {[string first "." [lindex $args 0]] == 0} {
            set widgetName [lindex $args 0]
            set args [lreplace $args 0 0]
        }
	set f1 $widgetName
	
	#
	# Set Command line option defaults here, 
	#   height, width and local count are freebies

	set sideTogState(height) 5;
	set sideTogState(width) 20;
	set sideTogState(iconWidth) 38;
	set sideTogState(pageWidth) 300;
	
	set argList {-width width
		-height height
		-iconWidth iconWidth
		-pageWidth pageWidth
		}
	
        # Scan the args for flags that need special handling
	# height, width  are examples
	#  Remove these entries if they are found.


        foreach {keyName indexName} $argList {
            if {[set pos [lsearch $args $keyName ]] >= 0} {
                set val [expr $pos + 1]
                set sideTogState($f1.$indexName) [lindex $args $val]
                set args [lreplace $args $pos $val]
            }
        }
                
        # Create master Frame
        
        set f1 [frame $widgetName -height $sideTogState(height) \
	    -width $sideTogState(width)]
	
	# Apply invocation options to the master frame, as appropriate

	foreach {opt val} $args {
	   catch {$f1 configure $opt $val}
	}

        ;#v vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv
	#; CREATE THE TK WIDGETS HERE, using whatever settings from cmd line

        # EXAMPLE FOR LABEL:
	# label $f1.l -text $sideTogState($f1.text)
	# set sideTogState(subWidgetName.label) l
	# pack $f1.l -side left
	
	canvas $f1.icons -height $sideTogState($f1.height) \
	    -width $sideTogState($f1.iconWidth) \
	    -relief ridge -borderwidth 2

	canvas $f1.pages -height $sideTogState($f1.height) \
	    -width $sideTogState($f1.pageWidth) \
	    -relief ridge -borderwidth 2
	    
	grid $f1.icons -row 0 -column 0 -sticky news
	grid $f1.pages -row 0 -column 1 -sticky news
	
	grid rowconfigure $f1    0 -weight 1
	grid columnconfigure $f1 1 -weight 1
        
	update ; update idle;

        ;#^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^

	debugPuts "PAST WIDGET CREATION"

        # We can't have two widgets with the same name.  
        #  Rename the base frame procedure for this
        #  widget to $widgetName.fr so that we can use
        #  $widgetName as the widget procedure name
        #  for the megawidget.

        uplevel #0 rename $widgetName $widgetName.fr
        
	set body [namespace code "sideToggleProc $widgetName \$args"]
	regsub -all "{" $body {"} body
	regsub -all "}" $body {"} body

        uplevel #0 [list proc $widgetName {args} "return \[$body\]"]

        #
        # Set any options that were defined in the command line.
        #
        # Process args as a list to keep from losing the second word of
	#  two word options

        ;#v vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv vv
	;# Any widgets that should accept individual command line options
	;# should be processed in this list
	
	# Example for the previous label

	# set createdWidgetList [list .l]
        set createdWidgetList [list .icons .pages]

        processOptions [list $args] $createdWidgetList $widgetName
		
        ;#^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^ ^^

	# When this window is destroyed, destroy the associated command.

	bind $widgetName <Destroy> "+[namespace code [list rename $widgetName {}]]"
        return $widgetName
    }
    
    proc GetBottom {cvs} {
	set l [$cvs find all]
	if {[llength $l ] > 0} {
          set bbox [eval $cvs bbox [$cvs find all]]
	  set bottom [lindex $bbox 3]
	} else {
	  set bottom 0
	}
        return [expr $bottom + 5]
    }

    proc addIcon {f1 iconImage} {
        variable sideTogState

	set newCvs [canvas $f1.pages.c$sideTogState(unique) \
  	    -height $sideTogState($f1.height) \
	    -width $sideTogState($f1.pageWidth) ]

	set newFram [frame $newCvs.f$sideTogState(unique)]
	$newCvs create window 0 0 -window $newFram -anchor nw
	$f1.pages create window 0 0 -window $newCvs -anchor nw

	incr sideTogState(unique)
	
	set bottom [GetBottom $f1.icons]

	set obj [$f1.icons create image 5 $bottom -image $iconImage -anchor nw]

	$f1.icons bind $obj <Button-1> "raise $newCvs"
	return $newFram
    }
    
    ##########################################################
    #     proc sideToggleProc {widgetName args}
    #    The master procedure for handling this megaWidget's
    #    subcommands
    # Arguments
    #   widgetName:          The name of the sideToggle
    #                        widget
    #   args:          The rest of the command line 
    #                               arguments
    #                        The first element must be the
    #                         subcommand name Subsequent list
    #                         elements depend on subcommand
    #     configure - configure entire widget, or certain
    #               subwidgets
    #               (configure subwidgetList -option1 \
    #                     val1 -option2 val2 ...)
    #     subwidget - return the full pathname of a 
    #               requested subwidget
    #     itemconfigure - configure a specific subwidget in this widget.
    #		mega itemconfigure itemID -key value
    #
    # Results
    #   Evaluates a subcommand and returns a result if required.
    #
    proc sideToggleProc {widgetName subCommand args} {
        variable sideTogState
        set cmdArgs $args
	
	# These are freebies:

	# megawid configure -flag Value ...
	# megawid cget subwidgetName -flag
	# megawid itemconfigure name -flag Value
        # megawid widgetnames << Returns list of names
        # megawid subwidget name << Returns valid name to access this subwidget

        switch -- $subCommand {
        itemconfigure {
                catch {eval $widgetName.$sideTogState(subWidgetName.[lindex $cmdArgs 0])  configure [lrange $cmdArgs 1 end]}
		}
        itemcget {
	        if {[llength $cmdArgs != 2]} {
		    error "$widgetName cget subWidgetName -option"
		}
                catch {eval $widgetName.$sideTogState(subWidgetName.[lindex $cmdArgs 0])  cget [lindex $cmdArgs 1]}
		}
        configure {
	        foreach n [array names sideTogState subWidgetName.*] {
		    lappend names $sideTogState($n)
		}
                makeOptionList $cmdArgs $names \
                       subwidgetList options
                set rtn [processOptions $options \
                        $subwidgetList $widgetName]
                return $rtn
            }
        widgetnames {
	        foreach n [array names sideTogState subWidgetName.*] {
		    foreach {d name} [split $n .] {}
		    lappend names $name
		}
	        return  $names
             }
        subwidget {
	        set name [lindex $cmdArgs 0]
	        if {[info exists sideTogState(subWidgetName.$name)]} {
		    return $widgetName.$sideTogState(subWidgetName.$name)
		}
             }
	add {
	    return [addIcon $widgetName $cmdArgs]
	}
        default {
                error "bad command: $subCommand -\
                    must be configure itemconfigure itemcget widgetnames\
                    or subwidget"
            }
        }
    }
}



if {[info exists argv] &&($argv == "-test")} {

proc testME {} {


set flasks0a { 
R0lGODlhIAAgAKIAANnZ2QAAADOZmQD///8AAMwAAJmZmQAA/yH5BAEAAAAALAAA
AAAgACAAAAP/CLrc/jDKSatFoYGgy+2NQkU0EHS5XVGIyAyKRiDocjeGCI6uSGAg
6HIvhsgMiuqMYgSCLndgyAyKrs6gRCDoMgeKzKiqUmhGIhB0WQNlBkWXd1AkEHQZ
AmUGRZd7UCIQdDcChaRKsKRKCkV3RjAQdDcCibB0FUM3RFAmAkFXI5AISzcCQRVD
dQJBdyOwiKgKIxB0OQYDQRdDlzsQdCECA0E30HQXQxdBV4kXhS53IOgCEQaCLkRg
BMAMBkBEIOhuMKNQADi8AIjAQFDF0FUE1bBAwNCFMAwE1ZhBGYkABIAIwwAMXQjD
CAAAjBkZwUBQDQsMgDDDAIAwDACAwBkZWwkE1bBA0DALBA0LVRQzMoIBABAYFgCA
YRYACGEBgABBIzgSAIAQhqHLChgWAIBBVFQYAACBFhBouhGBEBYACEFUhBUAABgK
qqoKQUQAIGDoBoIutz+MctL6UAIAOw==
}

image create photo flasks0a -data $flasks0a
unset flasks0a

set atom1b { 
R0lGODdhIAAgAKIAAAAAAP+ZAP//AP8AAP///wAA/wD/AP///ywAAAAAIAAgAAAD
/wi63P4wykmrvReFIuiyCoog6PIGgkaoqkIiACAIuqyBoBq7KAiqsrsLBYJuLKAg
6MruLhQIuhCKgqALwswACgRdCEHQBWFmCgWC7szgIOjK7q6FAgBgcIhmAAFAEHS5
AQOqqnCUxgwNBEGX19DMogipamZwAEAMzQxBV80QFKIKi2qmsApAAAHADEHVdrGi
qgqniqhQEHTBFg1BFyqwtGAVBUFXbdEQdKECq2ZUURB01RiQGYKqTCDRVBVKQQAg
gBmCrqKpwgzG0JRgAaCpIujyCoKZWaDpGgSCLjegAMwMTgiCbuzuWkgAAAZnJBB0
IZiZQoKgKxoIurG7CyBB0DmVBQwE3djdhQRBVXYxEFRjdxcSAABB0NVgDARdVkEA
EVVVCkUABIIur6Ag6DIHBoIutz+MctJKUQIAOw==
}

image create photo atom1b -data $atom1b

unset atom1b

    set eb [sideToggle -height 200 -iconWidth 35 -pageWidth 200]
    pack $eb

    set fr [$eb add atom1b]
    label $fr.top -text "Physics Lab" -font {helvetica 18 bold}
    grid $fr.top -row 0 -column 0

    set fr [$eb add flasks0a]

    label $fr.top -text "Chemistry Lab" -font {helvetica 18 bold}
    grid $fr.top -row 0 -column 0
    
}
testME
}

